# Generated by Django 5.2.11 on 2026-02-12 23:03

import django_extensions.db.fields.json
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0041_interface_mtu'),
    ]

    operations = [
        migrations.AddField(
            model_name='attribute',
            name='_default',
            field=django_extensions.db.fields.json.JSONField(blank=True, db_column='default', default=None, help_text='Default value applied when this attribute is not provided on resource creation.', null=True, verbose_name='Default'),
        ),
    ]
