"""App configuration for nsot."""

from django.apps import AppConfig


class NsotConfig(AppConfig):
    """Default configuration for nsot."""

    name = "nsot"
    verbose_name = "Network Source of Truth"
