# Generated by Django 5.2.11 on 2026-02-12 08:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0040_index_together_to_indexes'),
    ]

    operations = [
        migrations.AddField(
            model_name='interface',
            name='mtu',
            field=models.IntegerField(blank=True, db_index=True, default=None, help_text='MTU (Maximum Transmission Unit) in bytes.', null=True),
        ),
    ]
