# Generated by Django 5.2.11 on 2026-02-16 18:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0042_attribute__default'),
    ]

    operations = [
        migrations.AddField(
            model_name='circuit',
            name='expires_at',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='Optional expiration timestamp. Resources with expires_at in the past are considered expired.', null=True),
        ),
        migrations.AddField(
            model_name='device',
            name='expires_at',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='Optional expiration timestamp. Resources with expires_at in the past are considered expired.', null=True),
        ),
        migrations.AddField(
            model_name='interface',
            name='expires_at',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='Optional expiration timestamp. Resources with expires_at in the past are considered expired.', null=True),
        ),
        migrations.AddField(
            model_name='network',
            name='expires_at',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='Optional expiration timestamp. Resources with expires_at in the past are considered expired.', null=True),
        ),
        migrations.AddField(
            model_name='protocol',
            name='expires_at',
            field=models.DateTimeField(blank=True, db_index=True, default=None, help_text='Optional expiration timestamp. Resources with expires_at in the past are considered expired.', null=True),
        ),
    ]
