# Generated by Django 3.0.10 on 2020-09-23 12:24

from django.db import migrations, models
import django_extensions.db.fields.json
import nsot.fields


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0038_make_interface_speed_nullable'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='resource_name',
            field=models.CharField(choices=[('Network', 'Network'), ('Device', 'Device'), ('Interface', 'Interface'), ('Circuit', 'Circuit'), ('Protocol', 'Protocol')], db_index=True, help_text='The name of the Resource to which this Attribute is bound.', max_length=20, verbose_name='Resource Name'),
        ),
        migrations.AlterField(
            model_name='change',
            name='resource_name',
            field=models.CharField(choices=[('Site', 'Site'), ('Network', 'Network'), ('Attribute', 'Attribute'), ('Device', 'Device'), ('Interface', 'Interface'), ('Circuit', 'Circuit'), ('Protocol', 'Protocol'), ('ProtocolType', 'ProtocolType')], db_index=True, help_text='The name of the Resource for this Change.', max_length=20, verbose_name='Resource Type'),
        ),
        migrations.AlterField(
            model_name='interface',
            name='_addresses_cache',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=[]),
        ),
        migrations.AlterField(
            model_name='interface',
            name='_networks_cache',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=[]),
        ),
        migrations.AlterField(
            model_name='interface',
            name='mac_address',
            field=nsot.fields.MACAddressField(blank=True, db_index=True, default=0, help_text='If not provided, defaults to 0.', integer=True, null=True, verbose_name='MAC Address'),
        ),
        migrations.AlterField(
            model_name='interface',
            name='type',
            field=models.IntegerField(choices=[(6, 'ethernet'), (1, 'other'), (135, 'l2vlan'), (136, 'l3vlan'), (161, 'lag'), (24, 'loopback'), (150, 'mpls'), (53, 'prop_virtual'), (131, 'tunnel')], db_index=True, default=6, help_text="If not provided, defaults to 'ethernet'.", verbose_name='Interface Type'),
        ),
        migrations.AlterField(
            model_name='value',
            name='resource_name',
            field=models.CharField(choices=[('Network', 'Network'), ('Device', 'Device'), ('Interface', 'Interface'), ('Circuit', 'Circuit'), ('Protocol', 'Protocol')], db_index=True, help_text='The name of the Resource type to which the Value is bound.', max_length=20, verbose_name='Resource Type'),
        ),
    ]
