# Generated by Django 5.2.11 on 2026-02-17 06:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0043_circuit_expires_at_device_expires_at_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='attribute',
            name='depends_on',
            field=models.ManyToManyField(blank=True, help_text='Attributes that must also be present on a resource when this Attribute is set.', related_name='dependents', to='nsot.attribute'),
        ),
    ]
